package view;

import configuration.SimulationConfiguration;
import configuration.SimulationConfigurationSingleton;
import configuration.SimulationData;
import metadata.BuildingMetadata;
import simulation.Component;
import simulation.*;

import javax.imageio.ImageIO;
import javax.swing.*;
import java.awt.*;
import java.io.File;
import java.io.IOException;
import java.io.Serial;
import java.util.*;

public class MainPanel extends JPanel {
    @Serial
    private static final long serialVersionUID = 1L;

    private final Map<String, Image> buildingIcons = new HashMap<>();
    private final Map<ComponentType, Image> componentsIcons = new HashMap<>();
    private final Map<Integer, Building> buildingsById = new HashMap<>();
    private final Collection<ComponentRoute> componentRoutes = new ArrayList<>();

    private SimulationConfiguration configuration;
    private SimulationData simulationData;

    public MainPanel() {
        try {
            componentsIcons.put(ComponentType.METAL, loadImage("src/ressources/metal.png"));
            componentsIcons.put(ComponentType.MOTOR, loadImage("src/ressources/moteur.png"));
            componentsIcons.put(ComponentType.PLANE, loadImage("src/ressources/avion.png"));
            componentsIcons.put(ComponentType.WING, loadImage("src/ressources/aile.png"));
        } catch (IOException e) {
            throw new RuntimeException("Could not load components icons", e);
        }
    }

    public void reloadSimulation() {
        configuration = SimulationConfigurationSingleton.getInstance().getConfiguration();
        simulationData = configuration.getSimulationData();

        buildingsById.clear();
        simulationData.getBuildings().forEach(b -> buildingsById.put(b.getId(), b));

        buildingIcons.clear();
        configuration.getMetadata().values().forEach(m -> {
            try {
                loadBuildingImage(m);
            } catch (IOException e) {
                System.err.println("Could not load simulation icons");
            }
        });
    }

    @Override
    public void paint(Graphics g) {
        super.paint(g);

        if (configuration == null) return;

        simulationData.getRoutes().forEach(r -> paintRoute(r, g));
        simulationData.getBuildings().forEach(b -> paintBuilding(b, g));

        for (ComponentRoute route : componentRoutes) {
            paintComponentRoute(route, g);

            // TODO: Not working
            if (route.isTransitFinished()) {
                componentRoutes.remove(route);
            }
        }
        componentRoutes.forEach(r -> paintComponentRoute(r, g));
    }

    private void paintBuilding(Building building, Graphics g) {
        building.update().ifPresent(component -> addComponentRoute(component, building));

        Image icon = buildingIcons.get(building.getType());
        g.drawImage(icon, building.getX(), building.getY(), null);
    }

    private void addComponentRoute(Component component, Building fromBuilding) {
        Optional<Route> oroute = simulationData.getRoutes().stream()
                .filter(r -> r.getFrom() == fromBuilding.getId())
                .findFirst();

        Route route = oroute.get();
        Building toBuilding = buildingsById.get(route.getTo());
        componentRoutes.add(new ComponentRoute(component, toBuilding, fromBuilding.getX(), fromBuilding.getY()));
    }

    private void paintRoute(Route route, Graphics g) {
        Building fromBuilding = buildingsById.get(route.getFrom());
        Image fromIcon = buildingIcons.get(fromBuilding.getType());

        Building toBuilding = buildingsById.get(route.getTo());
        Image toIcon = buildingIcons.get(toBuilding.getType());

        g.drawLine(
                fromBuilding.getX() + fromIcon.getWidth(null) / 2,
                fromBuilding.getY() + fromIcon.getHeight(null) / 2,
                toBuilding.getX() + toIcon.getWidth(null) / 2,
                toBuilding.getY() + toIcon.getHeight(null) / 2
        );
    }

    private void paintComponentRoute(ComponentRoute route, Graphics g) {
        ComponentType componentType = route.getComponent().getType();
        Image componentIcon = componentsIcons.get(componentType);

        route.updatePosition();

        g.drawImage(componentIcon, route.getX(), route.getY(), null);
    }

    private void loadBuildingImage(BuildingMetadata metadata) throws IOException {
        String iconPath = metadata.getIconsPaths().get(BuildingState.EMPTY);
        Image image = loadImage(iconPath);

        buildingIcons.put(metadata.getType(), image);
    }

    private Image loadImage(String path) throws IOException {
        return ImageIO.read(new File(path));
    }
}
